﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using Loki;
using Loki.Common;
using Newtonsoft.Json;

namespace Scheduler
{
    /// <summary>Settings for the Scheduler. </summary>
    public class SchedulerSettings : JsonSettings
    {
        private static SchedulerSettings _instance;
        
        /// <summary>The current instance for this class. </summary>
        public static SchedulerSettings Instance
        {
            get { return _instance ?? (_instance = new SchedulerSettings()); }
        }

        /// <summary>The default ctor. Will use the settings path "Scheduler".</summary>
        public SchedulerSettings()
            : base(GetSettingsFilePath(Configuration.Instance.Name, string.Format("{0}.json", "Scheduler")))
        {
            // Setup defaults here if needed for properties that don't support DefaultValue.

			if (BotTimeLimit == default(TimeSpan))
			{
				BotTimeLimit = TimeSpan.FromHours(8);
			}

            if (BotTimeLimitMin == default(TimeSpan))
            {
				BotTimeLimitMin = TimeSpan.FromHours(8);
            }

			if (BotTimeLimitMax == default(TimeSpan))
			{
				BotTimeLimitMax = TimeSpan.FromHours(8);
			}

            if (BotIdleTimeLimitMin == default(TimeSpan))
            {
				BotIdleTimeLimitMin = TimeSpan.FromHours(4);
            }

			if (BotIdleTimeLimitMax == default(TimeSpan))
			{
				BotIdleTimeLimitMax = TimeSpan.FromHours(4);
			}
        }

        private bool _waitingForIdle;

        private readonly Stopwatch _programRuntimeStopwatch = new Stopwatch();
        private readonly Stopwatch _botRuntimeStopwatch = new Stopwatch();
        private readonly Stopwatch _botIdleStopwatch = new Stopwatch();

		private TimeSpan _botTimeLimit;
		private TimeSpan _botIdleTimeLimit;

        private bool _limitBotTime;
        private TimeSpan _botTimeLimitMin;
		private TimeSpan _botTimeLimitMax;
        private bool _startBotAfterLimitHit;
        private TimeSpan _botIdleTimeLimitMin;
		private TimeSpan _botIdleTimeLimitMax;

		/// <summary>
		/// The rolled time limit.
		/// </summary>
		[JsonIgnore]
		public TimeSpan BotTimeLimit
		{
			get { return _botTimeLimit; }
			set
			{
				if (value.Equals(_botTimeLimit))
				{
					return;
				}
				_botTimeLimit = value;
				NotifyPropertyChanged(() => BotTimeLimit);
			}
		}

		/// <summary>
		/// The rolled idle time limit
		/// </summary>
		[JsonIgnore]
		public TimeSpan BotIdleTimeLimit
		{
			get { return _botIdleTimeLimit; }
			set
			{
				if (value.Equals(_botIdleTimeLimit))
				{
					return;
				}
				_botIdleTimeLimit = value;
				NotifyPropertyChanged(() => BotIdleTimeLimit);
			}
		}

        [JsonIgnore]
        internal bool WaitingForIdle
        {
            get { return _waitingForIdle; }
            set
            {
                if (value.Equals(_waitingForIdle))
                {
                    return;
                }
                _waitingForIdle = value;
                NotifyPropertyChanged(() => WaitingForIdle);
            }
        }

        /// <summary>
        /// Should the bot be restarted after the bot time limit has been hit?
        /// </summary>
        [DefaultValue(false)]
        public bool StartBotAfterLimitHit
        {
            get { return _startBotAfterLimitHit; }
            set
            {
                if (value.Equals(_startBotAfterLimitHit))
                {
                    return;
                }
                _startBotAfterLimitHit = value;
                NotifyPropertyChanged(() => StartBotAfterLimitHit);
            }
        }

        /// <summary>
        /// The min time the bot should idle before starting again after the time limit is hit.
        /// </summary>
        public TimeSpan BotIdleTimeLimitMin
        {
            get { return _botIdleTimeLimitMin; }
            set
            {
				if (value.Equals(_botIdleTimeLimitMin))
                {
                    return;
                }
				_botIdleTimeLimitMin = value;
				NotifyPropertyChanged(() => BotIdleTimeLimitMin);
            }
        }

		/// <summary>
		/// The max time the bot should idle before starting again after the time limit is hit.
		/// </summary>
		public TimeSpan BotIdleTimeLimitMax
		{
			get { return _botIdleTimeLimitMax; }
			set
			{
				if (value.Equals(_botIdleTimeLimitMax))
				{
					return;
				}
				_botIdleTimeLimitMax = value;
				NotifyPropertyChanged(() => BotIdleTimeLimitMax);
			}
		}

        /// <summary>
        /// Should the time the bot is running be limited?
        /// </summary>
        [DefaultValue(false)]
        public bool LimitBotTime
        {
            get { return _limitBotTime; }
            set
            {
                if (value.Equals(_limitBotTime))
                {
                    return;
                }
                _limitBotTime = value;
                NotifyPropertyChanged(() => LimitBotTime);
            }
        }

        /// <summary>
        /// The min time the bot should run.
        /// </summary>
        public TimeSpan BotTimeLimitMin
        {
			get { return _botTimeLimitMin; }
            set
            {
				if (value.Equals(_botTimeLimitMin))
                {
                    return;
                }
				_botTimeLimitMin = value;
				NotifyPropertyChanged(() => BotTimeLimitMin);
            }
        }

		/// <summary>
		/// The max time the bot should run.
		/// </summary>
		public TimeSpan BotTimeLimitMax
		{
			get { return _botTimeLimitMax; }
			set
			{
				if (value.Equals(_botTimeLimitMax))
				{
					return;
				}
				_botTimeLimitMax = value;
				NotifyPropertyChanged(() => BotTimeLimitMax);
			}
		}

        [JsonIgnore]
        internal Stopwatch BotIdleStopwatch
        {
            get { return _botIdleStopwatch; }
        }

        [JsonIgnore]
        internal Stopwatch BotRuntimeStopwatch
        {
            get { return _botRuntimeStopwatch; }
        }

        [JsonIgnore]
        internal Stopwatch ProgramRuntimeStopwatch
        {
            get { return _programRuntimeStopwatch; }
        }

        /// <summary>
        /// The current program runtime.
        /// </summary>
        [JsonIgnore]
        public TimeSpan ProgramRuntime
        {
            get { return _programRuntimeStopwatch.Elapsed; }
            set
            {
                NotifyPropertyChanged(() => ProgramRuntime);
            }
        }

        /// <summary>
        /// The current bot runtime.
        /// </summary>
        [JsonIgnore]
        public TimeSpan BotRuntime
        {
            get { return _botRuntimeStopwatch.Elapsed; }
            set
            {
                NotifyPropertyChanged(() => BotRuntime);
            }
        }

        /// <summary>
        /// The current bot idle time.
        /// </summary>
        [JsonIgnore]
        public TimeSpan BotIdleTime
        {
            get { return _botIdleStopwatch.Elapsed; }
            set
            {
                NotifyPropertyChanged(() => BotIdleTime);
            }
        }
    }
}
